==> typestat-0.5.3.tgz: package/src/mutators/builtIn/fixStrictNonNullAssertions/fixStrictNonNullAssertionPropertyAccesses/index.ts <==
import { IMutation } from "automutate";
import * as ts from "typescript";

import { isTypeFlagSetRecursively } from "../../../../mutations/collecting/flags";
import { createNonNullAssertion } from "../../../../mutations/typeMutating/createNonNullAssertion";
import { getTypeAtLocationIfNotError } from "../../../../shared/types";
import { collectMutationsFromNodes } from "../../../collectMutationsFromNodes";
import { FileMutationsRequest, FileMutator } from "../../../fileMutator";

export const fixStrictNonNullAssertionPropertyAccesses: FileMutator = (request: FileMutationsRequest): ReadonlyArray<IMutation> => {
    const visitPropertyAccessExpression = (node: ts.PropertyAccessExpression): IMutation | undefined => {
        return getStrictPropertyAccessFix(request, node);
    };

    return collectMutationsFromNodes(request, ts.isPropertyAccessExpression, visitPropertyAccessExpression);
};

const getStrictPropertyAccessFix = (request: FileMutationsRequest, node: ts.PropertyAccessExpression): IMutation | undefined => {
    // Early on skip checking for "!" needs if there already is one
    if (ts.isAssertionExpression(node.parent) || ts.isNonNullExpression(node.parent)) {
        return undefined;
    }

    // Grab the type of the property being accessed by name
    const expressionType = getTypeAtLocationIfNotError(request, node.expression);

    // If the property's type cannot be null or undefined, rejoice! Nothing to do.
    if (expressionType === undefined || !isTypeFlagSetRecursively(expressionType, ts.TypeFlags.Null | ts.TypeFlags.Undefined)) {
        return undefined;
    }

    // Add a mutation to insert a "!" before the access
    return createNonNullAssertion(request, node);
};

==> typestat-0.5.4.tgz: package/src/mutators/builtIn/fixStrictNonNullAssertions/fixStrictNonNullAssertionPropertyAccesses/index.ts <==
import { Mutation } from "automutate";
import * as ts from "typescript";

import { isTypeFlagSetRecursively } from "../../../../mutations/collecting/flags";
import { createNonNullAssertion } from "../../../../mutations/typeMutating/createNonNullAssertion";
import { getTypeAtLocationIfNotError } from "../../../../shared/types";
import { collectMutationsFromNodes } from "../../../collectMutationsFromNodes";
import { FileMutationsRequest, FileMutator } from "../../../fileMutator";

export const fixStrictNonNullAssertionPropertyAccesses: FileMutator = (request: FileMutationsRequest): ReadonlyArray<Mutation> => {
    const visitPropertyAccessExpression = (node: ts.PropertyAccessExpression): Mutation | undefined => {
        return getStrictPropertyAccessFix(request, node);
    };

    return collectMutationsFromNodes(request, ts.isPropertyAccessExpression, visitPropertyAccessExpression);
};

const getStrictPropertyAccessFix = (request: FileMutationsRequest, node: ts.PropertyAccessExpression): Mutation | undefined => {
    // Early on skip checking for "!" needs if there already is one
    if (ts.isAssertionExpression(node.parent) || ts.isNonNullExpression(node.parent)) {
        return undefined;
    }

    // Grab the type of the property being accessed by name
    const expressionType = getTypeAtLocationIfNotError(request, node.expression);

    // If the property's type cannot be null or undefined, rejoice! Nothing to do.
    if (expressionType === undefined || !isTypeFlagSetRecursively(expressionType, ts.TypeFlags.Null | ts.TypeFlags.Undefined)) {
        return undefined;
    }

    // Add a mutation to insert a "!" before the access
    return createNonNullAssertion(request, node);
};
